
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import com.grapecity.documents.excel.HorizontalAlignment;
import com.grapecity.documents.excel.IWorksheet;
import com.grapecity.documents.excel.Workbook;

import oracle.jdbc.OracleConnection;

public class DSTTimeZoneIssueWithExcel {

	private static final String defaultPath = "N:/Workspaces/orbit-ws/eclipsews/TestProject/target/resources/";
	private static final String defaultFileName = "DataOutput_";
	private static final String longDateFormat = "yyyy-mm-dd hh:mm:ss";
	private static final String generalFormat = "@";

	private static final String driverClassName = "oracle.jdbc.OracleDriver";
	private static final String url = "jdbc:oracle:thin:@localhost:1521:XE";
	private static final String username = "orbfwl";
	private static final String password = "orbfwl";

	private static final String query = "select "
			+ "    obj_key, "
			+ "    to_date(to_char(created_date,'YYYYMMDD-HH:MI:SS'),'YYYYMMDD-HH:MI:SS') as save_date, "
			+ "    to_date(to_char(last_modified_date,'YYYYMMDD-HH:MI:SS'),'YYYYMMDD-HH:MI:SS') as edit_date, parent_id "
			+ "from ("
			+ "    SELECT "
			+ "        id, obj_key, created_date, last_modified_date, parent_id "
			+ "    FROM "
			+ "        orb_meta_object WHERE obj_type = 4 ORDER BY obj_key"
			+ "    ) tmptbl "
			+ "where "
			+ "    rownum < 8";

	public static void main(String[] args) {

		System.out.println();
		System.out.println(" -- Program Started --");
		System.out.println();
		
		System.out.println("Connecting to : " + url);
		System.out.println();
		
		boolean populateWorkbook = false;
		String fileName = null;
		
		if (args == null || args.length == 0) {
			System.out.println("Usage Guidelines ..");
			System.out.println("\tUse Y or N (Default) to write data to Excel or not");
			System.out.println("\tWhen Y is used, File Name (Optional) to write data can be provided");
		} else {

			if (args.length > 0 && "y".equalsIgnoreCase(args[0])) {
				populateWorkbook = true;
			}

			if (args.length > 1) {
				fileName = args[1].trim();
			}
		}

		System.out.println();

		process(populateWorkbook, fileName);

		System.out.println();
		System.out.println(" -- Program Completed --");
	}

	private static void process(boolean populateWorkbook, String fileName) {
		Connection conn = null;
		String timeZoneID = null;
		try {
			System.out.println("Getting connection ..");
			timeZoneID = TimeZone.getDefault().getID();
			conn = getConnection();
			System.out.println("Got connection");
			System.out.println();

			String dbTimeZone = ((OracleConnection) conn).getSessionTimeZone();
			System.out.println("Time Zone on DB Connection : " + dbTimeZone);

			/* Uncomment below block to use a different time zone */
			
			 timeZoneID = "MET";
			 ((OracleConnection)conn).setSessionTimeZone(timeZoneID);
			 dbTimeZone = ((OracleConnection)conn).getSessionTimeZone();
			 System.out.println("Changed Time Zone on DB Connection : " + dbTimeZone);

			TimeZone.setDefault(TimeZone.getTimeZone(timeZoneID));
			System.out.println();
			System.out.println("Time Zone on JVM           : " + timeZoneID);
			System.out.println();

			evaluateDateDiff(populateWorkbook, fileName, conn, timeZoneID);

		} catch (Exception exception) {
			exception.printStackTrace();
		} finally {
			if (conn != null) {
				try {
					conn.close();
					System.out.println("Connection closed");
				} catch (Exception exp) {
					System.out.println("Closing connection failed");
				}
			}
		}
	}

	private static void evaluateDateDiff(boolean populateWorkbook, String fileName, Connection conn, String timeZoneID) {
		

		System.out.println("Timezone Info .. ");
		System.out.println("Timezone Display Name : " + TimeZone.getDefault().getDisplayName());
		System.out.println("Timezone ID           : " + TimeZone.getDefault().getID());
		System.out.println("DST Savings           : " + TimeZone.getDefault().getDSTSavings());
		System.out.println("In DST                : " + TimeZone.getDefault().inDaylightTime(new Date()));
		System.out.println("Is DST Observed       : " + TimeZone.getDefault().observesDaylightTime());

		System.out.println();
		System.out.println("Local Info .. ");
		System.out.println("Locale Country        : " + Locale.getDefault().getCountry());
		System.out.println("Locale Language       : " + Locale.getDefault().getLanguage());

		System.out.println();
		System.out.println("Current Time out of [new Date()] : " + new Date());
		System.out.println();

		String tempPrefix = timeZoneID;
		tempPrefix = tempPrefix.replace("/", "-");
		if (fileName == null || fileName.length() <= 0) {
			fileName = defaultPath + tempPrefix + "_" + defaultFileName
					+ ((int) (Math.random() * ((2000 - 1000) + 1)) + 1000) + ".xlsx";
		} else {
			fileName = defaultPath + tempPrefix + "_" + fileName;
		}

		Workbook workbook = null;

		if (populateWorkbook) {
			workbook = prepareWorkbook();
		}

		processData(workbook, populateWorkbook, conn);

		if (populateWorkbook) {
			closeWorkbook(workbook, fileName);
		}


	}

	private static Workbook prepareWorkbook() {

		System.out.println("Opening Workbook");

		Workbook workbook = new Workbook();

		System.out.println("Workboook Opened");
		System.out.println();

		return workbook;
	}

	private static IWorksheet prepareWorksheet(Workbook workbook) throws SQLException {
		IWorksheet worksheet = workbook.getWorksheets().get(0);

		worksheet.getRange("A1").setValue("Object Key");

		worksheet.getRange("B1").setValue("Parent ID");

		worksheet.getRange("J1").setValue("F C Object");

		worksheet.getRange("X1").setValue("GF C Date");

		worksheet.getRange("A:A").setColumnWidthInPixel(250);
		worksheet.getRange("C:C").setColumnWidthInPixel(100);
		worksheet.getRange("E:E").setColumnWidthInPixel(125);
		worksheet.getRange("G:G").setColumnWidthInPixel(125);

		worksheet.getRange("A1:G1").getStyle().setHorizontalAlignment(HorizontalAlignment.Center);
		worksheet.getRange("A1:G1").getFont().setBold(true);
		worksheet.getRange("A1:G1").getFont().setItalic(true);

		worksheet.getRange("B:B").setColumnWidthInPixel(15);
		worksheet.getRange("D:D").setColumnWidthInPixel(15);
		worksheet.getRange("F:F").setColumnWidthInPixel(15);

		return worksheet;
	}

	private static void processData(Workbook workbook, boolean populateWorkbook, Connection conn) {
		System.out.println("Process of data started .. ");
		System.out.println();
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		Calendar utcCal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));

		try {

			pStmt = conn.prepareStatement(query);
			System.out.println("Statement created");
			rs = pStmt.executeQuery();
			System.out.println("Query executed");

			IWorksheet worksheet = null;
			if (populateWorkbook) {
				worksheet = prepareWorksheet(workbook);
			}

			System.out.println();

			long rowNumber = 2;
			System.out.println("Data .. ---- ");
			System.out.println();
			while (rs.next()) {

				if (populateWorkbook) {
					writeRecord(rs, utcCal, worksheet, rowNumber);
				} else {
					printRecord(rs, utcCal);
				}

				rowNumber++;
			}

			if (populateWorkbook) {
				formatWorksheet(worksheet);
			}

			System.out.println();
			System.out.println("End of Data  .. ---- ");
			System.out.println();
		} catch (Exception exception) {
			exception.printStackTrace();
		} finally {
			if (rs != null) {
				try {
					rs.close();
					System.out.println("Resultset closed");
				} catch (Exception exp) {
					System.out.println("Closing resultset failed");
				}
			}
			if (pStmt != null) {
				try {
					pStmt.close();
					System.out.println("Preparedstatement closed");
				} catch (Exception exp) {
					System.out.println("Closing statement failed");
				}
			}
			System.out.println();
			System.out.println("Process of data completed .. ");
			System.out.println();
		}
	}

	private static void formatWorksheet(IWorksheet worksheet) {

		worksheet.getRange("E:E").setNumberFormat(longDateFormat);

		worksheet.getRange("G:G").setNumberFormat(generalFormat);

	}

	private static void closeWorkbook(Workbook workbook, String fileName) {
		System.out.println("Savinf workbook at : " + fileName);
		workbook.save(fileName);
		System.out.println("Workbook Saved");
		System.out.println();
	}

	private static void writeRecord(ResultSet rs, Calendar utcCal, IWorksheet worksheet, long row) throws SQLException {

		worksheet.getRange("A" + row).setValue(rs.getObject(1));

		worksheet.getRange("C" + row).setValue(rs.getObject(4));

		worksheet.getRange("E" + row).setValue(rs.getObject(2));

		worksheet.getRange("G" + row).setValue(rs.getObject(2));

		printRecord(rs, utcCal);

	}

	private static void printRecord(ResultSet rs, Calendar utcCal) throws SQLException {

		System.out.println("getDataType\t" + rs.getString(1) + "\t" +rs.getInt(4) + "\t" +rs.getDate(2));
		System.out.println("getObject\t" + rs.getObject(1) + "\t" +rs.getObject(4) + "\t" +rs.getObject(2));
		System.out.println("-- --");

	}

	private static Connection getConnection() throws SQLException, ClassNotFoundException {

		Connection conn = null;

		Class.forName(driverClassName);

		conn = DriverManager.getConnection(url, username, password);

		return conn;

	}
}
